#include "Room.h"


Person::Person(int r, int c, Room* rm)
{
	row = r;
	col = c;
	room = rm;
	moved = false;
}


//Move a person closer to the exit
//For this function, each of the 9 squares the person is in
//	and is adjacent to is given a unique number as indicated below:
//			3 2 1
//			4 8 0
//			5 6 7
void Person::Move()
{
	//The below are initialized to the person's location, since a person should
	// only not move if every adjacent square is either occupied or further
	// from the exit
	int closest_col=col;	//column of closest adjacent square
	int closest_row=row;	//row of closest adjacent square
	//Distance from closest adjacent square to exit
	int closest_distance = room->exitDistance(closest_row, closest_col);

	//Examine each adjacent square.  Work counter-clockwise from down-right to
	// right to find the closest square to the exit
	int square=7;
	while(square>=0)
	{//For each adjacent square
		int col_test, row_test;
		switch(square)//Set square being examined
		{
			case 7: col_test = col+1;	//Down-right
					row_test = row+1;
					break;
			case 6: col_test = col;		//Down
					row_test = row+1;
					break;
			case 5: col_test = col-1;	//Down-left
					row_test = row+1;
					break;
			case 4: col_test = col-1;	//Left
					row_test = row;
					break;
			case 3: col_test = col-1;	//Up-left
					row_test = row-1;
					break;
			case 2: col_test = col;		//Up
					row_test = row-1;
					break;
			case 1: col_test = col+1;	//Up-right
					row_test = row-1;
					break;
			case 0: col_test = col+1;	//Right
					row_test = row;
					break;
		}
		if(room->legalSquare(row_test, col_test)==true)
		{//If square being tested is in the room
			int test_distance = room->exitDistance(row_test, col_test);

			if((room->Occupied(row_test, col_test) == false) && 
				(test_distance <= closest_distance))
			{//If square being tested is unoccupied and closer than current
			 // closest square
				//Update closest square
				closest_col=col_test;
				closest_row=row_test;
				closest_distance = test_distance;
			}
		}
		square--;
	}

	//Move person to closest square to exit, if movement is required
	if((closest_row != row) || (closest_col != col))
	{
		room->addPerson(this, closest_row, closest_col);
		room->removePerson(row, col);
		row = closest_row;
		col = closest_col;
	}

	//Set moved true
	moved = true;
}
